<?php

namespace Ems\Repository;

use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;

use Ems\Util\Utils;

class ApuracaoRepository {
    /** @var Connection */
    protected $connection;

    public function __construct(
        EntityManager       $entityManager,
        PeriodoRepository   $periodoRepository
    ) {
        $this->connection           = $entityManager->getConnection();
        $this->periodoRepository    = $periodoRepository;
    }

    public function retrocederApuracao(
        array $params
    ) : int {

        $sql = <<<SQL
            DELETE FROM
                PTO_RESUMOSITUFREQ
            WHERE NRORG = :P_NRORG
              AND NRVINCULOM    = :P_NRVINCULOM
              AND NRPERIODOAPURACAO   = :P_NRPERIODOAPURACAO
              AND DTAPURACAOSITUFREQ >= TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY')
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $qntDeletedRows = $std->rowCount();

        return $qntDeletedRows;
    }

    public function getResumoSituFreqVinculo(
        array $params
    ) : array {

        $sql = <<<SQL
            SELECT *
            FROM PTO_RESUMOSITUFREQ
            WHERE NRORG = :P_NRORG
              AND NRVINCULOM = :P_NRVINCULOM
              AND NRPERIODOAPURACAO = :P_NRPERIODOAPURACAO
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetchAll();

        return $result;
    }
    
    public function getResumoDiaSituFreqVinculo(
        array $params
    ) : array {

        $sql = <<<SQL
            SELECT *
            FROM PTO_RESUMOSITUFREQ
            WHERE NRORG                                     = :P_NRORG
              AND NRVINCULOM                                = :P_NRVINCULOM
              AND TO_CHAR(DTAPURACAOSITUFREQ, 'DD/MM/YYYY') = :P_DTAPURACAOSITUFREQ
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetchAll();

        return $result;
    }

    public function getAlteEscala(
        array $params
    ) : array {

        $sql = <<<SQL
            SELECT *
            FROM GPE_ALTEESCALA
            WHERE NRORG = :P_NRORG
              AND NRVINCULOM = :P_NRVINCULOM
              AND TO_DATE(:P_DATE,'DD/MM/YYYY')
                BETWEEN DTINIESCALA
                AND NVL(DTFIMESCALA, TO_DATE(:P_DATE,'DD/MM/YYYY'))
            ORDER BY DTINIESCALA DESC
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetchAll();

        return $result;
    }

    public function deletarIntervalosPeriodo(
        array $params
    ) : int {

        $sql = <<<SQL
            DELETE FROM PTO_INTERVALOSMARCACAO
               WHERE NRORG = :P_NRORG
                    AND NRINTERVALOSMARCACAO IN (SELECT NRINTERVALOSMARCACAO
                                              FROM PTO_INTERVALOSMARCACAO
                                                JOIN GPE_MARCACAOPONTO MARCACAOINI
                                                    ON MARCACAOINI.NRORG           = PTO_INTERVALOSMARCACAO.NRORG
                                                AND MARCACAOINI.NRMARCACAOPONTO = PTO_INTERVALOSMARCACAO.NRMARCACAOPONTOINI
                                                JOIN GPE_MARCACAOPONTO MARCACAOFIM
                                                    ON MARCACAOFIM.NRORG           = PTO_INTERVALOSMARCACAO.NRORG
                                                AND MARCACAOFIM.NRMARCACAOPONTO = PTO_INTERVALOSMARCACAO.NRMARCACAOPONTOFIM
                                                JOIN PTO_PERIODOAPURACAO
                                                    ON PTO_PERIODOAPURACAO.NRORG = PTO_INTERVALOSMARCACAO.NRORG
                                                AND TRUNC(MARCACAOINI.DTHRAJUSTADA) >= TRUNC(PTO_PERIODOAPURACAO.DTINICIALAPURACAO)
                                                AND TRUNC(MARCACAOINI.DTHRAJUSTADA) <= TRUNC(PTO_PERIODOAPURACAO.DTFINALAPURACAO)
                                                AND TRUNC(MARCACAOFIM.DTHRAJUSTADA) >= TRUNC(PTO_PERIODOAPURACAO.DTINICIALAPURACAO)
                                                AND TRUNC(MARCACAOFIM.DTHRAJUSTADA) <= TRUNC(PTO_PERIODOAPURACAO.DTFINALAPURACAO)
                                              WHERE PTO_INTERVALOSMARCACAO.NRORG = :P_NRORG
                                                AND MARCACAOINI.NRVINCULOM       = :P_NRVINCULO
                                                AND NRPERIODOAPURACAO            = :P_NRPERIODO)
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $qntDeletedRows = $std->rowCount();

        return $qntDeletedRows;
    }

    public function deletarResumoPeriodo(
        array $params
    ) : int {

        $sql = <<<SQL
            DELETE FROM
                PTO_RESUMOSITUFREQ
            WHERE NRORG = :P_NRORG
              AND NRVINCULOM    = :P_NRVINCULOM
              AND NRPERIODOAPURACAO   = :P_NRPERIODOAPURACAO
              AND DTAPURACAOSITUFREQ >= TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY')
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $qntDeletedRows = $std->rowCount();

        return $qntDeletedRows;
    }

    public function deleteMarcacoesPeriodo($nrorg, $nrvinculom, $nrperiodoapuracao) {
      $sql = <<<SQL
            DELETE FROM GPE_MARCACAOPONTO
                      WHERE NRORG = :P_NRORG
                        AND NRMARCACAOPONTO IN (SELECT NRMARCACAOPONTO
                                                    FROM GPE_MARCACAOPONTO
                                                    JOIN PTO_PERIODOAPURACAO
                                                      ON PTO_PERIODOAPURACAO.NRORG              = GPE_MARCACAOPONTO.NRORG
                                                     AND TRUNC(GPE_MARCACAOPONTO.DTHRAJUSTADA) >= TRUNC(PTO_PERIODOAPURACAO.DTINICIALAPURACAO)
                                                     AND TRUNC(GPE_MARCACAOPONTO.DTHRAJUSTADA) <= TRUNC(PTO_PERIODOAPURACAO.DTFINALAPURACAO)
                      JOIN GPE_PARAMORG
                        ON GPE_PARAMORG.NRORG                = GPE_MARCACAOPONTO.NRORG
                      AND (GPE_PARAMORG.NRCOLETORMARCGERSIST   = GPE_MARCACAOPONTO.NRCOLETOR OR
                           GPE_PARAMORG.NRCOLETORMARCPROJETADA = GPE_MARCACAOPONTO.NRCOLETOR OR
                           GPE_PARAMORG.NRCOLETORMARCVIRADIA   = GPE_MARCACAOPONTO.NRCOLETOR OR
                           GPE_PARAMORG.NRCOLETORMARCAUTOM     = GPE_MARCACAOPONTO.NRCOLETOR)
                     WHERE GPE_MARCACAOPONTO.NRORG             = :P_NRORG
                       AND GPE_MARCACAOPONTO.NRVINCULOM        = :P_NRVINCULOM
                       AND NRPERIODOAPURACAO                   = :P_NRPERIODOAPURACAO)
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute(
            array(
                'P_NRORG' => $nrorg,
                'P_NRVINCULOM' => $nrvinculom,
                'P_NRPERIODOAPURACAO' => $nrperiodoapuracao
            )
        );
    }
    
    public function deleteMarcacoesDia($nrorg, $nrvinculom, $dia) {
      $sql = <<<SQL
            DELETE FROM GPE_MARCACAOPONTO
                      WHERE NRORG = :P_NRORG
                        AND NRMARCACAOPONTO IN (SELECT NRMARCACAOPONTO
                                                    FROM GPE_MARCACAOPONTO
                                                    JOIN PTO_PERIODOAPURACAO
                                                      ON PTO_PERIODOAPURACAO.NRORG                              = GPE_MARCACAOPONTO.NRORG
                                                     AND TO_CHAR(GPE_MARCACAOPONTO.DTHRAJUSTADA, 'DD/MM/YYYY')  = :P_DTAPURACAOPONTO
                      JOIN GPE_PARAMORG
                        ON GPE_PARAMORG.NRORG                = GPE_MARCACAOPONTO.NRORG
                      AND (GPE_PARAMORG.NRCOLETORMARCGERSIST   = GPE_MARCACAOPONTO.NRCOLETOR OR
                           GPE_PARAMORG.NRCOLETORMARCPROJETADA = GPE_MARCACAOPONTO.NRCOLETOR OR
                           GPE_PARAMORG.NRCOLETORMARCVIRADIA   = GPE_MARCACAOPONTO.NRCOLETOR OR
                           GPE_PARAMORG.NRCOLETORMARCAUTOM     = GPE_MARCACAOPONTO.NRCOLETOR)
                     WHERE GPE_MARCACAOPONTO.NRORG             = :P_NRORG
                       AND GPE_MARCACAOPONTO.NRVINCULOM        = :P_NRVINCULOM)
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute(
            array(
                'P_NRORG'           => $nrorg,
                'P_NRVINCULOM'      => $nrvinculom,
                'P_DTAPURACAOPONTO' => $dia
            )
        );
    }

    public function deleteIntervalosPeriodo($nrorg, $nrvinculom, $nrperiodoapuracao) {
        $sql = <<<SQL
            DELETE FROM PTO_INTERVALOSMARCACAO
              WHERE NRORG = :P_NRORG
                AND NRINTERVALOSMARCACAO IN (
                 SELECT NRINTERVALOSMARCACAO
                   FROM PTO_INTERVALOSMARCACAO
                   JOIN GPE_MARCACAOPONTO MARCACAOINI
                     ON MARCACAOINI.NRORG           = PTO_INTERVALOSMARCACAO.NRORG
                    AND MARCACAOINI.NRMARCACAOPONTO = PTO_INTERVALOSMARCACAO.NRMARCACAOPONTOINI
                   JOIN GPE_MARCACAOPONTO MARCACAOFIM
                     ON MARCACAOFIM.NRORG           = PTO_INTERVALOSMARCACAO.NRORG
                    AND MARCACAOFIM.NRMARCACAOPONTO = PTO_INTERVALOSMARCACAO.NRMARCACAOPONTOFIM
                   JOIN PTO_PERIODOAPURACAO
                     ON PTO_PERIODOAPURACAO.NRORG = PTO_INTERVALOSMARCACAO.NRORG
                    AND TRUNC(MARCACAOINI.DTHRAJUSTADA) >= TRUNC(PTO_PERIODOAPURACAO.DTINICIALAPURACAO)
                    AND TRUNC(MARCACAOINI.DTHRAJUSTADA) <= TRUNC(PTO_PERIODOAPURACAO.DTFINALAPURACAO)
                    AND TRUNC(MARCACAOFIM.DTHRAJUSTADA) >= TRUNC(PTO_PERIODOAPURACAO.DTINICIALAPURACAO)
                    AND TRUNC(MARCACAOFIM.DTHRAJUSTADA) <= TRUNC(PTO_PERIODOAPURACAO.DTFINALAPURACAO)
                  WHERE PTO_INTERVALOSMARCACAO.NRORG = :P_NRORG
                    AND MARCACAOINI.NRVINCULOM       = :P_NRVINCULOM
                    AND NRPERIODOAPURACAO            = :P_NRPERIODOAPURACAO
                )
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute(
            array(
                'P_NRORG' => $nrorg,
                'P_NRVINCULOM' => $nrvinculom,
                'P_NRPERIODOAPURACAO' => $nrperiodoapuracao
            )
        );
    }
    
    public function deleteIntervalosDia($nrorg, $nrvinculom, $dia) {
        $sql = <<<SQL
            DELETE FROM PTO_INTERVALOSMARCACAO
              WHERE NRORG = :P_NRORG
                AND NRINTERVALOSMARCACAO IN (
                 SELECT NRINTERVALOSMARCACAO
                   FROM PTO_INTERVALOSMARCACAO
                   JOIN GPE_MARCACAOPONTO MARCACAOINI
                     ON MARCACAOINI.NRORG           = PTO_INTERVALOSMARCACAO.NRORG
                    AND MARCACAOINI.NRMARCACAOPONTO = PTO_INTERVALOSMARCACAO.NRMARCACAOPONTOINI
                   JOIN GPE_MARCACAOPONTO MARCACAOFIM
                     ON MARCACAOFIM.NRORG           = PTO_INTERVALOSMARCACAO.NRORG
                    AND MARCACAOFIM.NRMARCACAOPONTO = PTO_INTERVALOSMARCACAO.NRMARCACAOPONTOFIM
                    AND TO_CHAR(MARCACAOINI.DTHRAJUSTADA, 'DD/MM/YYY') = :P_DTAPURACAOPONTO
                    AND TO_CHAR(MARCACAOFIM.DTHRAJUSTADA, 'DD/MM/YYY') = :P_DTAPURACAOPONTO
                  WHERE PTO_INTERVALOSMARCACAO.NRORG = :P_NRORG
                    AND MARCACAOINI.NRVINCULOM       = :P_NRVINCULOM
                )
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute(
            array(
                'P_NRORG'           => $nrorg,
                'P_NRVINCULOM'      => $nrvinculom,
                'P_DTAPURACAOPONTO' => $dia
            )
        );
    }

    public function deleteResumosPeriodo($nrorg, $nrvinculom, $nrperiodoapuracao) {
        $sql = <<<SQL
                DELETE FROM PTO_RESUMOSITUFREQ
                      WHERE NRORG             = :P_NRORG
                        AND NRVINCULOM        = :P_NRVINCULOM
                        AND NRPERIODOAPURACAO = :P_NRPERIODOAPURACAO
SQL;

      $std = $this->connection->prepare($sql);
        $std->execute(
            array(
                'P_NRORG' => $nrorg,
                'P_NRVINCULOM' => $nrvinculom,
                'P_NRPERIODOAPURACAO' => $nrperiodoapuracao
            )
        );
    }
    
    public function deleteResumosdia($nrorg, $nrvinculom, $dtapuracaositufreq) {
        $sql = <<<SQL
                DELETE FROM PTO_RESUMOSITUFREQ
                      WHERE NRORG                                       = :P_NRORG
                        AND NRVINCULOM                                  = :P_NRVINCULOM
                        AND TO_CHAR(DTAPURACAOSITUFREQ, 'DD/MM/YYYY')   = :P_DTAPURACAOSITUFREQ
SQL;

      $std = $this->connection->prepare($sql);
        $std->execute(
            array(
                'P_NRORG' => $nrorg,
                'P_NRVINCULOM' => $nrvinculom,
                'P_DTAPURACAOSITUFREQ' => $dtapuracaositufreq
            )
        );
    }

    public function updateMarcacoesAlteradasPeriodo($nrorg, $nrvinculom, $nrperiodoapuracao) {
      $sql = <<<SQL
                  UPDATE GPE_MARCACAOPONTO
                   SET IDMARCDESPREZADA  = 'N',
                       IDENQUADRAHORARIO = 'N'
                 WHERE NRORG = :P_NRORG
                    AND NRMARCACAOPONTO IN (SELECT NRMARCACAOPONTO
                                             FROM GPE_MARCACAOPONTO
                                             JOIN PTO_PERIODOAPURACAO
                                               ON PTO_PERIODOAPURACAO.NRORG       = GPE_MARCACAOPONTO.NRORG
                                              AND TRUNC(GPE_MARCACAOPONTO.DTHRAJUSTADA) >= TRUNC(PTO_PERIODOAPURACAO.DTINICIALAPURACAO)
                                              AND TRUNC(GPE_MARCACAOPONTO.DTHRAJUSTADA) <= TRUNC(PTO_PERIODOAPURACAO.DTFINALAPURACAO)
                                            WHERE GPE_MARCACAOPONTO.NRORG             = :P_NRORG
                                              AND GPE_MARCACAOPONTO.NRVINCULOM        = :P_NRVINCULOM
                                              AND NRPERIODOAPURACAO                   = :P_NRPERIODOAPURACAO)
SQL;

      $std = $this->connection->prepare($sql);
        $std->execute(
            array(
                'P_NRORG' => $nrorg,
                'P_NRVINCULOM' => $nrvinculom,
                'P_NRPERIODOAPURACAO' => $nrperiodoapuracao
            )
        );
    }
    
    public function updateMarcacoesAlteradasDia($nrorg, $nrvinculom, $nrperiodoapuracao) {
      $sql = <<<SQL
                  UPDATE GPE_MARCACAOPONTO
                   SET IDMARCDESPREZADA  = 'N',
                       IDENQUADRAHORARIO = 'N'
                 WHERE NRORG = :P_NRORG
                    AND NRMARCACAOPONTO IN (SELECT NRMARCACAOPONTO
                                             FROM GPE_MARCACAOPONTO
                                             JOIN PTO_PERIODOAPURACAO
                                               ON PTO_PERIODOAPURACAO.NRORG                             = GPE_MARCACAOPONTO.NRORG
                                              AND TO_CHAR(GPE_MARCACAOPONTO.DTHRAJUSTADA, 'DD/MM/YYYY') = :P_DTAPURACAOSITUFREQ
                                            WHERE GPE_MARCACAOPONTO.NRORG                               = :P_NRORG
                                              AND GPE_MARCACAOPONTO.NRVINCULOM                          = :P_NRVINCULOM)
SQL;

      $std = $this->connection->prepare($sql);
        $std->execute(
            array(
                'P_NRORG' => $nrorg,
                'P_NRVINCULOM' => $nrvinculom,
                'P_DTAPURACAOSITUFREQ' => $nrperiodoapuracao
            )
        );
    }

    public function updateNovoCodigoResumos($nrorg) {
      $sql = <<<SQL
                  UPDATE NOVOCODIGO
                   SET NRSEQUENCIAL = (SELECT LPAD(MAX(NRRESUMOSITUFREQ) + 1,'20','0')
                                         FROM PTO_RESUMOSITUFREQ
                                        WHERE NRORG  = :P_NRORG
                                        )
                 WHERE NRORG      = :P_NRORG
                   AND CDCONTADOR = 'PTO_RESUMOSITUFREQ'
SQL;

      $std = $this->connection->prepare($sql);
        $std->execute(
            array(
                'P_NRORG' => $nrorg
            )
        );
    }

    public function updateNovoCodigoMarcacoes($nrorg) {
      $sql = <<<SQL
                  UPDATE NOVOCODIGO
                   SET NRSEQUENCIAL = (SELECT LPAD(MAX(NRMARCACAOPONTO) + 1,'20','0')
                                         FROM GPE_MARCACAOPONTO
                                        WHERE NRORG  = :P_NRORG
                                        )
                 WHERE NRORG      = :P_NRORG
                   AND CDCONTADOR = 'GPE_MARCACAOPONTO'
SQL;

      $std = $this->connection->prepare($sql);
        $std->execute(
            array(
                'P_NRORG' => $nrorg
            )
        );
    }

    public function updateNovoCodigoIntervalos($nrorg) {
      $sql = <<<SQL
                  UPDATE NOVOCODIGO
                   SET NRSEQUENCIAL = (SELECT LPAD(MAX(NRINTERVALOSMARCACAO) + 1,'20','0')
                                         FROM PTO_INTERVALOSMARCACAO
                                        WHERE NRORG  = :P_NRORG
                                        )
                 WHERE NRORG      = :P_NRORG
                   AND CDCONTADOR = 'PTO_INTERVALOSMARCACAO'

SQL;

      $std = $this->connection->prepare($sql);
        $std->execute(
            array(
                'P_NRORG' => $nrorg
            )
        );
    }
    
    public function buscaDiasComMarcacaoNaoApurados(
                $nrorg,
                $nrvinculom,
        string  $dataInicial,
        string  $dataFinal = null
    ) {
        $sql = <<<SQL
        
        SELECT
            TO_CHAR(DTHRMARCACAOPONTO, 'DD/MM/YYYY') DTHRMARCACAOPONTO
        FROM (
              SELECT
                  TRUNC(MP.DTHRMARCACAOPONTO) DTHRMARCACAOPONTO
              FROM
                  GPE_MARCACAOPONTO    MP
                  LEFT JOIN PTO_RESUMOSITUFREQ   RF ON RF.NRORG = MP.NRORG
                                                     AND RF.NRVINCULOM = MP.NRVINCULOM
                                                     AND RF.DTAPURACAOSITUFREQ = (
                                                         CASE
                                                            WHEN MP.IDDIAOCORRENCIA IN ('FA', 'DA') THEN
                                                                TRUNC(MP.DTHRAJUSTADA) - 1
                                                            WHEN MP.IDDIAOCORRENCIA IN ('FS', 'DS') THEN
                                                                TRUNC(MP.DTHRAJUSTADA) + 1
                                                            ELSE
                                                                TRUNC(MP.DTHRAJUSTADA)
                                                          END
                                                      )
              WHERE
                  MP.NRORG = :P_NRORG
                  AND MP.NRVINCULOM = :P_NRVINCULOM
                   AND MP.DTHRMARCACAOPONTO BETWEEN TO_DATE(:P_DATAINICIAL, 'DD/MM/YYYY') AND TO_DATE(:P_DATAFINAL, 'DD/MM/YYYY') + 1
                  AND MP.NRCOLETOR <> 21
                  
                  /* GARANTE QUE NÃO EXISTE RESUMO DE FREQUENCIA */
                  AND RF.NRRESUMOSITUFREQ IS NULL
                  
                  /* VERIFICA SE O DIA NÃO É FERIADO */
                  AND NOT EXISTS (
                      SELECT
                          1
                      FROM
                          OCORCALENDARIO
                      WHERE
                          NRCALENDARIO = (
                              SELECT
                                  NRCALENDARIO
                              FROM
                                  ESTRUTURAH
                              WHERE
                                  NRESTRUTURAM = (
                                      SELECT MOV.NRESTRUTURAM
                                      FROM GPE_MOVIMENTACAO MOV
                                      JOIN GPE_PARAMORG PO ON PO.NRORG = MOV.NRORG
                                      WHERE MOV.NRORG = :P_NRORG
                                          AND MOV.NRVINCULOM = :P_NRVINCULOM
                                          AND MOV.DTINIMOVIMENT <= LAST_DAY(TO_DATE(:P_DATAFINAL, 'DD/MM/YYYY'))
                                          AND (MOV.DTFIMMOVIMENT IS NULL OR MOV.DTFIMMOVIMENT >  LAST_DAY(TO_DATE(:P_DATAFINAL, 'DD/MM/YYYY')))
                                          AND MOV.NRTIPOESTRUTURA IN (PO.NRTPESTRUTLEGAL, PO.NRTPESTRUTCALEND)
                                      ORDER BY DECODE(MOV.NRTIPOESTRUTURA, PO.NRTPESTRUTCALEND, 1, PO.NRTPESTRUTLEGAL, 2, 3), MOV.DTINIMOVIMENT DESC
                                      FETCH FIRST ROW ONLY
                                  )
                                  AND DTMESCOMPETENC IN (
                                      SELECT
                                          MAX(DTMESCOMPETENC)
                                      FROM
                                          ESTRUTURAH H1
                                      WHERE
                                          H1.NRESTRUTURAM = ESTRUTURAH.NRESTRUTURAM
                                          AND H1.NRORG = ESTRUTURAH.NRORG
                                          AND H1.DTMESCOMPETENC <= TO_DATE(:P_DATAFINAL, 'DD/MM/YYYY')
                                  )
                                  AND NRORG = :P_NRORG
                          )
                          AND NRORG = :P_NRORG
                          AND TO_CHAR(DTCALENDARIO, 'DD/MM/YYYY') = TO_CHAR(MP.DTHRMARCACAOPONTO, 'DD/MM/YYYY')
                  )
                  
                  GROUP BY
                      TRUNC(DTHRMARCACAOPONTO)
                  ORDER BY
                      DTHRMARCACAOPONTO
            )
            
SQL;
        
        $std = $this->connection->prepare($sql);

        $std->execute(
            array(
                'P_NRORG'       => $nrorg,
                'P_NRVINCULOM'  =>  $nrvinculom,
                'P_DATAINICIAL' => $dataInicial,
                'P_DATAFINAL'   => $dataFinal,
            )    
        );

        $result = $std->fetchAll();

        return $result;
    }
    
    public function getDiasPosterioresNaoApurados(
                $nrorg,
                $nrvinculom,
        string  $data
    ) {
        $sql = <<<SQL
    SELECT TO_CHAR(RS.DTAPURACAOSITUFREQ, 'DD/MM/YYYY') AS DTAPURACAOSITUFREQ FROM GPE_MARCACAOPONTO MP
    JOIN GPE_VINCULOM           VM ON VM.NRORG = MP.NRORG AND VM.NRVINCULOM = MP.NRVINCULOM
    JOIN PTO_RESUMOSITUFREQ     RS ON RS.NRORG = MP.NRORG AND RS.NRVINCULOM = MP.NRVINCULOM AND TO_CHAR(RS.DTAPURACAOSITUFREQ, 'DD/MM/YYYY') = TO_CHAR(MP.DTHRMARCACAOPONTO, 'DD/MM/YYYY')
    JOIN PTO_PERIODOAPURACAO    PA ON PA.NRORG = MP.NRORG AND TO_DATE(:P_DATA, 'DD/MM/YYYY') BETWEEN PA.DTINICIALAPURACAO AND PA.DTFINALAPURACAO AND PA.NRTIPOPERIODOAPURACAO = (
        SELECT NRTIPOPERIODOAPURACAO FROM PTO_TIPOAPURACAOFREQ
        WHERE NRORG = MP.NRORG AND NRTIPOAPURACAO = VM.NRTIPOVINCULOM
    )
    WHERE MP.NRORG = :P_NRORG AND MP.NRVINCULOM = :P_NRVINCULOM AND RS.DTAPURACAOSITUFREQ BETWEEN TO_DATE(:P_DATA, 'DD/MM/YYYY') AND PA.DTFINALAPURACAO
    GROUP BY DTAPURACAOSITUFREQ
    ORDER BY DTAPURACAOSITUFREQ
SQL;

        $std = $this->connection->prepare($sql);

        $std->execute(
            array(
                'P_NRORG'       => $nrorg,
                'P_NRVINCULOM'  => $nrvinculom,
                'P_DATA'        => $data,
            )
        );

        $result = $std->fetchAll();

        return $result;
    }

    public function _getApuracaoPorData(
        string $nrvinculom,
        string $nrorg,
        string $nrperiodoapuracao,
        string $dtapuracaoponto,
        string $dtmescompetencia
    ) {

        /* retorna os resumos de situação de frequência, as marcações de entrada/saída, e ausências totalizadas  */
        $resumosSitufreq = $this->getResumosSitufreqPorData(
            $nrvinculom,
            $nrorg,
            $nrperiodoapuracao,
            $dtapuracaoponto,
            $dtmescompetencia
        );

        /* verifica se existe ocorrencia de frequência no dia para compensação de banco de horas */
        $ocorrenciasCompensarBH = $this->verificaOcorreciaCompensacaoBH(
            $nrvinculom,
            $nrorg,
            $dtapuracaoponto
        );

        /* verifica se existe ocorrencia de frequência no dia para autorização de hora extra e verifica o status da mesma */
        $ocorrenciasAutorizacaoEX = $this->verificaOcorrenciaAutorizacaoEX(
            $nrvinculom,
            $nrorg,
            $dtapuracaoponto
        );

        /* verifica se existe ocorrencia de frequência no dia para identificação de intervalos */
        $ocorrenciasIntervalos = $this->verificaIdentificaIntervalo(
            $nrvinculom,
            $nrorg,
            $dtapuracaoponto
        );

        /* verifica se existe ocorrencia de frequência no dia para abonar falta e verifica o status da mesma */
        $ocorrenciasAbonarFalta = $this->verificaAbonarFalta(
            $nrvinculom,
            $nrorg,
            $dtapuracaoponto
        );

        /* verifica se existe ocorrencia de frequência no dia para compensar falta e verifica o status da mesma */
        $ocorrenciasCompensarFalta = $this->verificaCompensarFalta(
            $nrvinculom,
            $nrorg,
            $dtapuracaoponto
        );

        /* verifica se existem ocorrencias pendentes para o vinculo no dia a ser apurado */
        $ocorrenciasPendentes = $this->verificaExisteOcorrenciaPendente(
            $nrvinculom,
            $nrorg,
            $dtapuracaoponto
        );

        /* Verifica se existem resumos de situação de frequência de faltas diurnas ou noturnas para o dia*/
        $faltasDiurnasNoturnas = $this->verificaExisteResumoFaltaDiurnaNoturna(
            $nrvinculom,
            $nrorg,
            $nrperiodoapuracao,
            $dtapuracaoponto
        );

        /* Verifica se existe ocorrências de feriado no calendário para a data */
        $ocorrenciasCalendario = $this->verificaOcorrenciasCalendario(
            $nrvinculom,
            $nrorg,
            $dtapuracaoponto,
            $dtmescompetencia
        );

        $apuracaoPonto = $this->factory(
            $nrvinculom,
            $nrorg,
            $nrperiodoapuracao,
            $dtapuracaoponto,
            $dtmescompetencia,
            $resumosSitufreq,
            $ocorrenciasCompensarBH,
            $ocorrenciasAutorizacaoEX,
            $ocorrenciasIntervalos,
            $ocorrenciasAbonarFalta,
            $ocorrenciasCompensarFalta,
            $ocorrenciasPendentes,
            $faltasDiurnasNoturnas,
            $ocorrenciasCalendario
        );

        return $apuracaoPonto;

    }
    
    public function getResumosSitufreqPorData(
        string $nrvinculom,
        string $nrorg,
        string $nrperiodoapuracao,
        string $dtapuracaoponto,
        string $dtmescompetencia
    ) {
        $sql = <<<SQL
            SELECT
                APU_DIA.*
            FROM TABLE (PONTODEMONST_DADOS(TO_DATE(:P_DTMESCOMPETENCIA, 'DD/MM/YYYY'), :P_NRORG, TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY'), :P_NRPERIODOAPURACAO, :P_NRVINCULOM)) APU_DIA
SQL;
        $query = $this->connection->prepare($sql);

        $query->execute(
            array(
                "P_DTMESCOMPETENCIA"  => $dtmescompetencia,
                "P_NRORG"             => $nrorg,
                "P_DTAPURACAOPONTO"   => $dtapuracaoponto,
                "P_NRPERIODOAPURACAO" => $nrperiodoapuracao,
                "P_NRVINCULOM"        => $nrvinculom
            )
        );

        $apuracaoPonto = $query->fetchAll();
        return $apuracaoPonto[0];
    }
    
    private function verificaOcorreciaCompensacaoBH(
        $nrvinculom,
        $nrorg,
        $dtapuracaoponto
    ) {
        $sql = <<<SQL
            SELECT
               COUNT(*) AS COUNT_COMPENSACAO_BH
              FROM PTO_PROGOCORRENCIA
             WHERE NRORG                 = :P_NRORG
               AND NRVINCULOM            = :P_NRVINCULOM
               AND IDATIVO               = 'S'
               AND NRTIPOPROGOCORRENCIA IN (2, 10, 18, 19)
               AND DTINICIOPROGOCOR      = TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY')
               AND EXISTS (SELECT 1
                             FROM GPE_MARCACAOPONTO
                            WHERE NRORG      = PTO_PROGOCORRENCIA.NRORG
                              AND NRVINCULOM = PTO_PROGOCORRENCIA.NRVINCULOM
                              AND TRUNC(DTHRMARCACAOPONTO) = TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY'))
SQL;
        $query = $this->connection->prepare($sql);

        $query->execute(
            array(
                "P_NRORG"           => $nrorg,
                "P_NRVINCULOM"      => $nrvinculom,
                "P_DTAPURACAOPONTO" => $dtapuracaoponto
            )
        );

        $ocorrencias = $query->fetchAll();
        return $ocorrencias[0];
    }
    
    private function verificaOcorrenciaAutorizacaoEX(
        string $nrvinculom,
        string $nrorg,
        string $dtapuracaoponto
    ) {
        $sql = <<<SQL
            SELECT
               COUNT(*) AS COUNT_AUTORIZACAO_EX,
               IDPROGOCORCONFIRMADA
              FROM PTO_PROGOCORRENCIA
             WHERE NRORG                = :P_NRORG
               AND NRVINCULOM           = :P_NRVINCULOM
               AND NRTIPOPROGOCORRENCIA = 1
               AND IDATIVO              = 'S'
               AND (DTINICIOPROGOCOR     = TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY')
                   OR DTFIMPROGOCOR IS NULL)
             GROUP BY IDPROGOCORCONFIRMADA
SQL;
        $query = $this->connection->prepare($sql);

        $query->execute(
            array(
                "P_NRORG"           => $nrorg,
                "P_NRVINCULOM"      => $nrvinculom,
                "P_DTAPURACAOPONTO" => $dtapuracaoponto
            )
        );

        $ocorrencias = $query->fetchAll();
        if (isset($ocorrencias[0])) {
            return $ocorrencias[0];
        } else {
            return null;
        }
    }
    
    private function verificaIdentificaIntervalo(
        string $nrvinculom,
        string $nrorg,
        string $dtapuracaoponto
    ) {
        $sql = <<<SQL
            SELECT COUNT(*) AS COUNT_IDENTIFICA_INTERVALO
              FROM PTO_PROGOCORRENCIA PROG
              JOIN GPE_MARCACAOPONTO MARC
                ON MARC.NRMARCACAOPONTO = PROG.NRMARCACAOPONTO
               AND MARC.NRVINCULOM      = PROG.NRVINCULOM
               AND MARC.NRORG           = PROG.NRORG
             WHERE PROG.NRORG                 = :P_NRORG
               AND PROG.NRVINCULOM            = :P_NRVINCULOM
               AND PROG.IDATIVO               = 'S'
               AND PROG.NRTIPOPROGOCORRENCIA IN (20, 26)
               AND PROG.DTINICIOPROGOCOR      = TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY')
               AND PROG.NRMARCACAOPONTO IS NOT NULL
SQL;
        $query = $this->connection->prepare($sql);

        $query->execute(
            array(
                "P_NRORG"           => $nrorg,
                "P_NRVINCULOM"      => $nrvinculom,
                "P_DTAPURACAOPONTO" => $dtapuracaoponto
            )
        );

        $ocorrencias = $query->fetchAll();
        if (isset($ocorrencias[0])) {
            return $ocorrencias[0];
        } else {
            return null;
        }
    }
    
    private function verificaAbonarFalta(
        string $nrvinculom,
        string $nrorg,
        string $dtapuracaoponto
    ) {
        $sql = <<<SQL
            SELECT
                COUNT(*) AS COUNT_ABONAR_FALTA,
                IDPROGOCORCONFIRMADA,
                IDTIPODIACALENDARIO,
                NRPROGOCORRENCIA,
                DSOBSERVACAO,
                DSOBSERVACAORECUSA
              FROM PTO_PROGOCORRENCIA PROG
             WHERE PROG.NRORG                = :P_NRORG
               AND PROG.NRVINCULOM           = :P_NRVINCULOM
               AND PROG.NRTIPOPROGOCORRENCIA = 5
               AND IDATIVO                   = 'S'
               AND TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY') BETWEEN PROG.DTINICIOPROGOCOR AND PROG.DTFIMPROGOCOR
               /*AND PROG.DTINICIOPROGOCOR     = TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY')
               AND PROG.DTFIMPROGOCOR        = TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY')*/
               GROUP BY IDPROGOCORCONFIRMADA, IDTIPODIACALENDARIO, NRPROGOCORRENCIA, DSOBSERVACAO, DSOBSERVACAORECUSA
SQL;
        $query = $this->connection->prepare($sql);

        $query->execute(
            array(
                "P_NRORG"           => $nrorg,
                "P_NRVINCULOM"      => $nrvinculom,
                "P_DTAPURACAOPONTO" => $dtapuracaoponto
            )
        );

        $ocorrencias = $query->fetchAll();
        if (isset($ocorrencias[0])) {
            return $ocorrencias[0];
        } else {
            return null;
        }
    }
    
    private function verificaCompensarFalta(
        string $nrvinculom,
        string $nrorg,
        string $dtapuracaoponto
    ) {
        $sql = <<<SQL
            SELECT
                COUNT(*) AS COUNT_COMPENSACAO_FALTA,
                IDPROGOCORCONFIRMADA,
                NRPROGOCORRENCIA,
                DSOBSERVACAO,
                DSOBSERVACAORECUSA
              FROM PTO_PROGOCORRENCIA PROG
             WHERE PROG.NRORG                = :P_NRORG
               AND PROG.NRVINCULOM           = :P_NRVINCULOM
               AND NRTIPOPROGOCORRENCIA IN (2, 10, 18, 19)
               AND IDATIVO              = 'S'
               AND TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY') BETWEEN PROG.DTINICIOPROGOCOR AND PROG.DTFIMPROGOCOR
               AND NOT EXISTS (SELECT 1
                                 FROM GPE_MARCACAOPONTO
                                WHERE NRORG      = PROG.NRORG
                                  AND NRVINCULOM = PROG.NRVINCULOM
                                  AND NRCOLETOR  != 21
                                  AND TRUNC(DTHRMARCACAOPONTO) = TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY'))
               GROUP BY IDPROGOCORCONFIRMADA, NRPROGOCORRENCIA, DSOBSERVACAO, DSOBSERVACAORECUSA
SQL;
        $query = $this->connection->prepare($sql);

        $query->execute(
            array(
                "P_NRORG"           => $nrorg,
                "P_NRVINCULOM"      => $nrvinculom,
                "P_DTAPURACAOPONTO" => $dtapuracaoponto
            )
        );

        $ocorrencias = $query->fetchAll();
        if (isset($ocorrencias[0])) {
            return $ocorrencias[0];
        } else {
            return null;
        }
    }
    
    
    public function verificaExisteOcorrenciaPendente(
        string $nrvinculom,
        string $nrorg,
        string $dtapuracaoponto
    ) {
        $sql = <<<SQL
            SELECT
                COUNT(*) AS COUNT_OCORRENCIAS_PENDENTES
              FROM PTO_PROGOCORRENCIA PROG
             WHERE PROG.NRORG                = :P_NRORG
               AND IDATIVO                   = 'S'
               AND PROG.NRVINCULOM           = :P_NRVINCULOM
               AND PROG.DTINICIOPROGOCOR     = TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY')
               AND PROG.IDPROGOCORCONFIRMADA = 'P'
SQL;
        $query = $this->connection->prepare($sql);

        $query->execute(
            array(
                "P_NRORG"           => $nrorg,
                "P_NRVINCULOM"      => $nrvinculom,
                "P_DTAPURACAOPONTO" => $dtapuracaoponto
            )
        );

        $ocorrenciasPendentes = $query->fetchAll();
        if (isset($ocorrenciasPendentes[0])) {
            return $ocorrenciasPendentes[0];
        } else {
            return null;
        }
    }
    
    public function verificaExisteResumoFaltaDiurnaNoturna(
        string $nrvinculom,
        string $nrorg,
        string $nrperiodoapuracao,
        string $dtapuracaoponto
    ) {
        $sql = <<<SQL
            SELECT
              COUNT(*) AS COUNT_FALTAS
             FROM PTO_RESUMOSITUFREQ RES
            WHERE NRORG              = :P_NRORG
              AND NRVINCULOM         = :P_NRVINCULOM
              AND NRPERIODOAPURACAO  = :P_NRPERIODOAPURACAO
              AND DTAPURACAOSITUFREQ = TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY')
              AND NRSITUFREQUENCIA  IN (10500 /*Falta Diurna*/, 10600 /*Falta Noturna*/)
SQL;
        $query = $this->connection->prepare($sql);

        $query->execute(
            array(
                "P_NRORG"             => $nrorg,
                "P_NRVINCULOM"        => $nrvinculom,
                "P_NRPERIODOAPURACAO" => $nrperiodoapuracao,
                "P_DTAPURACAOPONTO"   => $dtapuracaoponto
            )
        );

        $faltasEncontradas = $query->fetchAll();
        if (isset($faltasEncontradas[0])) {
            return $faltasEncontradas[0];
        } else {
            return null;
        }
    }
    
    public function verificaOcorrenciasCalendario(
        string $nrvinculom,
        string $nrorg,
        string $dtapuracaoponto,
        string $dtmescompetencia
    ) {
        $sql = <<<SQL
            SELECT
              COUNT(*) AS COUNT_OCORRENCIAS_FERIADO
              /*OCORCALENDARIO.DTCALENDARIO,
              OCORCALENDARIO.DSOBSCALEND,
              TPCALENDARIO.DSTPCALENDARIO,
              TPCALENDARIO.DSABREVTPCALEND,
              TPCALENDARIO.IDTIPODIA*/
             FROM OCORCALENDARIO
             JOIN TPCALENDARIO
               ON TPCALENDARIO.NRTPCALENDARIO = OCORCALENDARIO.NRTPCALENDARIO
            WHERE OCORCALENDARIO.NRORG = :P_NRORG
              AND OCORCALENDARIO.DTCALENDARIO = TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY')
              AND OCORCALENDARIO.NRCALENDARIO IN (SELECT NRCALENDARIO
                                    FROM ESTRUTURAH
                                   WHERE NRESTRUTURAM = (
                                        SELECT MOV.NRESTRUTURAM
                                        FROM GPE_MOVIMENTACAO MOV
                                        JOIN GPE_PARAMORG PO ON PO.NRORG = MOV.NRORG
                                        WHERE MOV.NRORG = :P_NRORG
                                            AND MOV.NRVINCULOM = :P_NRVINCULOM
                                            AND MOV.DTINIMOVIMENT <= LAST_DAY(TO_DATE(:P_DTMESCOMPETENCIA, 'DD/MM/YYYY'))
                                            AND (MOV.DTFIMMOVIMENT IS NULL OR MOV.DTFIMMOVIMENT >  LAST_DAY(TO_DATE(:P_DTMESCOMPETENCIA, 'DD/MM/YYYY')))
                                            AND MOV.NRTIPOESTRUTURA IN (PO.NRTPESTRUTLEGAL, PO.NRTPESTRUTCALEND)
                                        ORDER BY DECODE(MOV.NRTIPOESTRUTURA, PO.NRTPESTRUTCALEND, 1, PO.NRTPESTRUTLEGAL, 2, 3), MOV.DTINIMOVIMENT DESC
                                        FETCH FIRST ROW ONLY
                                   )
                                    AND DTMESCOMPETENC IN (
                                        SELECT MAX(DTMESCOMPETENC)
                                           FROM ESTRUTURAH H1
                                          WHERE H1.NRESTRUTURAM = ESTRUTURAH.NRESTRUTURAM
                                            AND H1.NRORG = ESTRUTURAH.NRORG
                                            AND H1.DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENCIA, 'DD/MM/YYYY')
                                    )
                                AND NRORG = :P_NRORG)
              AND OCORCALENDARIO.NRORG = :P_NRORG
SQL;
        $query = $this->connection->prepare($sql);

        $query->execute(
            array(
                "P_NRORG"            => $nrorg,
                "P_NRVINCULOM"       => $nrvinculom,
                "P_DTAPURACAOPONTO"  => $dtapuracaoponto,
                "P_DTMESCOMPETENCIA" => $dtmescompetencia
            )
        );

        $ocorrenciasCalendario = $query->fetchAll();
        if (isset($ocorrenciasCalendario[0])) {
            return $ocorrenciasCalendario[0];
        } else {
            return null;
        }
    }
    
    private function factory(
        $nrvinculom,
        $nrorg,
        $nrperiodoapuracao,
        $dtapuracaoponto,
        $dtmescompetencia,
        $resumosSitufreq,
        $ocorrenciasCompensarBH,
        $ocorrenciasAutorizacaoEX,
        $ocorrenciasIntervalos,
        $ocorrenciasAbonarFalta,
        $ocorrenciasCompensarFalta,
        $ocorrenciasPendentes,
        $faltasDiurnasNoturnas,
        $ocorrenciasCalendario
    ) {

        $apuracaoPonto = array();
        $apuracaoPonto["DTAPURACAOSITUFREQ"] = $dtapuracaoponto;

        if ($ocorrenciasCalendario['COUNT_OCORRENCIAS_FERIADO'] > 0) {
            $apuracaoPonto["OCORRENCIA_FERIADO"] = true;
        } else {
            $apuracaoPonto["OCORRENCIA_FERIADO"] = false;
        }

        if ($ocorrenciasPendentes['COUNT_OCORRENCIAS_PENDENTES'] > 0) {
            $apuracaoPonto["OCORRENCIAS_PENDENTES"] = true;
        } else {
            $apuracaoPonto["OCORRENCIAS_PENDENTES"] = false;
        }

        if ($faltasDiurnasNoturnas['COUNT_FALTAS'] > 0) {
            $apuracaoPonto["FALTA_IDENTIFICADA"] = true;
        } else {
            $apuracaoPonto["FALTA_IDENTIFICADA"] = false;
        }

        if ($ocorrenciasCompensarBH['COUNT_COMPENSACAO_BH'] > 0) {
            $apuracaoPonto["HORASDECONTADAS"]          = "00:00";                         /*diferença de horas a serem descontadas*/
            $apuracaoPonto["HORASBANCODIA"]            = $resumosSitufreq['BH_DESC_DIA']; /*diferença de horas a serem compensadas*/
            $apuracaoPonto["SOLICITOU_COMPENSACAO_BH"] = true;                            /*identificador de solicitação de hora extra*/
        } else {
            $apuracaoPonto["HORASDECONTADAS"]          = $resumosSitufreq['BH_DESC_DIA'];
            $apuracaoPonto["HORASBANCODIA"]            = "00:00";
            $apuracaoPonto["SOLICITOU_COMPENSACAO_BH"] = false;
        }

        if ($ocorrenciasAutorizacaoEX['COUNT_AUTORIZACAO_EX'] > 0) {
            $apuracaoPonto["HORASEXTRAS"]              = "00:00";                     /*horas extras não autorizadas*/
            $apuracaoPonto["HORASEXTRASAUTORIZADA"]    = $resumosSitufreq['HE_AUTO']; /*horas extras autorizadas*/

            if (floatval(str_replace(',', '.', $resumosSitufreq['HE_AUTO'])) > 0)
                $apuracaoPonto["SOLICITOU_HORAEXTRA"]      = true;                     /*identificador de solicitação de hora extra*/
            else
                $apuracaoPonto["SOLICITOU_HORAEXTRA"]      = false;                    /*identificador de solicitação de hora extra*/

            $apuracaoPonto["ID_SITUACAO_HE"]           = $ocorrenciasAutorizacaoEX['IDPROGOCORCONFIRMADA'];
        } else {
            $apuracaoPonto["HORASEXTRAS"]              = $resumosSitufreq['HE_NAUT'];
            $apuracaoPonto["HORASEXTRASAUTORIZADA"]    = "00:00";
            $apuracaoPonto["SOLICITOU_HORAEXTRA"]      = false;
            $apuracaoPonto["ID_SITUACAO_HE"]           = $ocorrenciasAutorizacaoEX['IDPROGOCORCONFIRMADA'];
        }

        if ($ocorrenciasIntervalos["COUNT_IDENTIFICA_INTERVALO"] > 0) {
            $apuracaoPonto["FALTA_MARCACAO_ALMOCO"] = false; /*identificador de identificação de intervalo*/
        } else {
            $apuracaoPonto["FALTA_MARCACAO_ALMOCO"] = true;
        }

        if ($ocorrenciasAbonarFalta["COUNT_ABONAR_FALTA"] > 0) {
            $apuracaoPonto["SOLICITOU_ABONAR_FALTA"]   = true;
            $apuracaoPonto["ID_SITUACAO_ABONARFALTA"]  = $ocorrenciasAbonarFalta['IDPROGOCORCONFIRMADA'];
            $apuracaoPonto["IDTIPODIACALENDARIO_OCOR"] = $ocorrenciasAbonarFalta['IDTIPODIACALENDARIO'];
            $apuracaoPonto["NRPROGOCORRENCIA_OCOR"]    = $ocorrenciasAbonarFalta['NRPROGOCORRENCIA'];
            $apuracaoPonto["DSOBSERVACAO_OCOR"]        = $ocorrenciasAbonarFalta['DSOBSERVACAO'];
            $apuracaoPonto["DSOBSERVACAORECUSA_OCOR"]  = $ocorrenciasAbonarFalta['DSOBSERVACAORECUSA'];
        } else {
            $apuracaoPonto["SOLICITOU_ABONAR_FALTA"]   = false;
            $apuracaoPonto["ID_SITUACAO_ABONARFALTA"]  = $ocorrenciasAbonarFalta['IDPROGOCORCONFIRMADA'];
            $apuracaoPonto["IDTIPODIACALENDARIO_OCOR"] = "";
        }

        if ($ocorrenciasCompensarFalta["COUNT_COMPENSACAO_FALTA"] > 0) {
            $apuracaoPonto["SOLICITOU_COMPENSAR_FALTA"]  = true;
            $apuracaoPonto["ID_SITUACAO_COMPENSARFALTA"] = $ocorrenciasCompensarFalta['IDPROGOCORCONFIRMADA'];
            $apuracaoPonto["NRPROGOCORRENCIA_OCOR"]      = $ocorrenciasCompensarFalta['NRPROGOCORRENCIA'];
            $apuracaoPonto["DSOBSERVACAO_OCOR"]          = $ocorrenciasAbonarFalta['DSOBSERVACAO'];
            $apuracaoPonto["DSOBSERVACAORECUSA_OCOR"]    = $ocorrenciasAbonarFalta['DSOBSERVACAORECUSA'];
        } else {
            $apuracaoPonto["SOLICITOU_COMPENSAR_FALTA"]  = false;
            $apuracaoPonto["ID_SITUACAO_COMPENSARFALTA"] = $ocorrenciasCompensarFalta['IDPROGOCORCONFIRMADA'];
        }

        // LEONARDOCARVALHO - 21/11/2017 - Calcula tempo de trabalho efetivo interno, total trabalhado subtraindo o tempo justificado
        $horastrabalhadasefetivas = floatval(str_replace(',', '.', $resumosSitufreq['TRABALH'])) - floatval(str_replace(',', '.', $resumosSitufreq['AU_JUST']));

        $apuracaoPonto["ENTRADA"]                  = $resumosSitufreq['ENTRADA'];      /*hora de entrada*/
        $apuracaoPonto["ALMOCO"]                   = $resumosSitufreq['ALMOCO'];       /*tempo de almoço*/
        $apuracaoPonto["SAIDA"]                    = $resumosSitufreq['SAIDA'];        /*hora de saída*/
        $apuracaoPonto["INJUSTIFICADAS"]           = $resumosSitufreq['AU_INJU'];      /*ausência intrajornada injustificada*/
        $apuracaoPonto["JUSTIFICADAS"]             = $resumosSitufreq['AU_JUST'];      /*ausência intrajornada justificada*/
        $apuracaoPonto["TOTAL_AUSENCIA"]           = $resumosSitufreq['AU_TOTA'];      /*ausência intrajornada tempo total*/
        $apuracaoPonto["HORASTRABALHADASEFETIVAS"]  = (String)$horastrabalhadasefetivas; /*horas trabalhadas normais/efetivas*/
        $apuracaoPonto["HORASTRABALHADAS"]         = $resumosSitufreq['TRABALH'];      /*horas trabalhadas com horas justificadas*/
        $apuracaoPonto["DIFERENCAJORNADA"]         = $resumosSitufreq['DIF_TRA'];      /*diferença sobre jornada*/
        $apuracaoPonto["SALDO_BH_ANTERIOR"]        = $resumosSitufreq['BH_DIA_ANT'];   /*banco de horas do dia anterior*/
        $apuracaoPonto["SALDO_DIA"]                = $resumosSitufreq['BH_DESC_DIA'];  /*banco de horas do dia*/
        $apuracaoPonto["BH_S_AC"]                  = $resumosSitufreq['BH_S_AC'];      /*banco de horas sem acréscimo*/
        $apuracaoPonto["SALDO_BH_ATUAL"]           = $resumosSitufreq['BH_C_AC'];      /*banco de horas com acréscimo*/
        $apuracaoPonto["FL_ATRA"]                  = $resumosSitufreq['FL_ATRA'];      /*faltas atrazos*/
        $apuracaoPonto["FL_DIAS"]                  = $resumosSitufreq['FL_DIAS'];      /*faltas dias*/
        $apuracaoPonto["HORASNORMAIS"]             = $resumosSitufreq['TRAB_NORM'];    /*horas base da jornada de trabalho*/

        return $apuracaoPonto;

    }
    
    public function getApuracaoPorData(
        string $nrvinculom,
        string $nrorg,
        string $dtapuracaoponto     = null,
        string $nrperiodoapuracao   = null,
        string $dtmescompetencia    = null
    ) { 
        if (empty($nrperiodoapuracao)) {
            $nrperiodoapuracao = $this->periodoRepository->findPeriodoByDia(
                $nrorg,
                $nrvinculom,
                $dtapuracaoponto
            );
            $nrperiodoapuracao = $nrperiodoapuracao['NRPERIODOAPURACAO'];
        }

        if (empty($dtmescompetencia)) {
            $dtmescompetencia = $this->periodoRepository->getDtmescompetencByPeriodo($nrorg, $nrperiodoapuracao);
        }

        return $this->_getApuracaoPorData(
            $nrvinculom,
            $nrorg,
            $nrperiodoapuracao,
            $dtapuracaoponto,
            $dtmescompetencia
        );
    }
    
    /**
     * Retorna vinculos e dias a serem apurados ate determinada data levando em consideracao detalhes como escala e datas de admissao/rescisao
     * 
     * @param string $nrPeriodoApuracao
     * @param string $dataInicio
     * @param array $nrvinculomList
     * @param array $nrestrutlegalList
     * @param array $nrestrutgerenList
     * @param array $nrtipovinculomList
     * @param string $dataApuracao
     * @return array
     */
    public function getVinculosPontoAtivoAteData(
        string $dataFim,
        ?array $nrvinculomList = [],
        ?array $nrestrutlegalList = [],
        ?array $nrestrutgerenList = [],
        ?array $nrtipovinculomList = []
    ): array {
        $sql = <<<SQL
            SELECT
              VM.NRORG,
              VM.NRVINCULOM,
              VM.NMVINCULOM,
              VM.NRTIPOVINCULOM,
              VM.DTADMISSAOVINC,
              VM.DTRESCISAOVINC,
              LISTAGG(TO_CHAR(DATAS_APURACAO.DTDATA, 'DD/MM/YYYY'), ',') WITHIN GROUP (ORDER BY DTDATA) DATAS_APURAR_PERIODO_ATUAL
            FROM GPE_VINCULOM VM
            JOIN GPE_VINCULOH VH ON VH.NRVINCULOM = VM.NRVINCULOM
                AND VH.NRORG = VM.NRORG
                AND VH.DTMESCOMPETENC IN (
                SELECT
                    MAX(DTMESCOMPETENC)
                FROM
                    GPE_VINCULOH VIH
                WHERE
                    VIH.NRVINCULOM = VH.NRVINCULOM
                    AND VIH.NRORG = VH.NRORG
                    AND TO_CHAR(VIH.DTMESCOMPETENC, 'DD/MM/YYYY') <= :P_DTMESCOMPETENC
            )
            JOIN PTO_PERIODOAPURACAO PA ON PA.NRORG = VM.NRORG
              AND :P_DATA_FIM BETWEEN PA.DTINICIALAPURACAO AND PA.DTFINALAPURACAO
              AND PA.NRTIPOPERIODOAPURACAO = (
                SELECT TP.NRTIPOPERIODOAPURACAO
                FROM PTO_TIPOPERIODOAPURACAO TP
                WHERE TP.NRORG = PA.NRORG
                  AND TP.NRTIPOVINCULOM = VM.NRTIPOVINCULOM
              )
            LEFT JOIN DATA DATAS_APURACAO ON TRUNC(DATAS_APURACAO.DTDATA) BETWEEN PA.DTINICIALAPURACAO AND TRUNC(LEAST(NVL(VM.DTRESCISAOVINC, TO_DATE(:P_DATA_FIM, 'DD/MM/YYYY')), TO_DATE(:P_DATA_FIM, 'DD/MM/YYYY')))
              AND PA.IDPERIODOAPURACAO <> 'FECHADO'
            WHERE VM.NRORG = :P_NRORG
              AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= PA.DTINICIALAPURACAO)
              AND EXISTS (
                  SELECT 1
                  FROM GPE_ALTEESCALA AE
                  WHERE AE.NRORG = VM.NRORG
                    AND AE.NRVINCULOM = VM.NRVINCULOM
                    AND AE.NRTURNO IS NOT NULL
                    AND AE.DTINIESCALA = (
                    SELECT
                        MAX(DTINIESCALA)
                    FROM
                        GPE_ALTEESCALA
                    WHERE
                        NRORG = AE.NRORG
                        AND NRVINCULOM = AE.NRVINCULOM
                        AND TRUNC(DATAS_APURACAO.DTDATA) BETWEEN TRUNC(DTINIESCALA) AND TRUNC(NVL(DTFIMESCALA, DATAS_APURACAO.DTDATA))
                )
              )
              AND ( INSTR(:P_NRVINCULOM_LIST, ',' || VM.NRVINCULOM || ',') > 0 OR :P_NRVINCULOM_LIST IS NULL )
              AND ( INSTR(:P_NRESTRUTLEGAL_LIST, ',' || VH.NRESTRUTLEGAL || ',') > 0 OR :P_NRESTRUTLEGAL_LIST IS NULL )
              AND ( INSTR(:P_NRESTRUTGEREN_LIST, ',' || VH.NRESTRUTGEREN || ',') > 0 OR :P_NRESTRUTGEREN_LIST IS NULL )
              AND ( INSTR(:P_NRTIPOVINCULOM_LIST, ',' || VM.NRTIPOVINCULOM || ',') > 0 OR :P_NRTIPOVINCULOM_LIST IS NULL )
            GROUP BY VM.NRORG, VM.NRVINCULOM, VM.NMVINCULOM, VM.DTADMISSAOVINC, VM.DTRESCISAOVINC, VM.NRTIPOVINCULOM
            ORDER BY NMVINCULOM
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute([
            'P_NRORG'               => Utils::getParamters()->getNrorg(),
            'P_DATA_FIM'            => $dataFim,
            'P_NRVINCULOM_LIST'     => Utils::arrayToStringForQueryParam($nrvinculomList),
            'P_NRESTRUTLEGAL_LIST'  => Utils::arrayToStringForQueryParam($nrestrutlegalList),
            'P_NRESTRUTGEREN_LIST'  => Utils::arrayToStringForQueryParam($nrestrutgerenList),
            'P_NRTIPOVINCULOM_LIST' => Utils::arrayToStringForQueryParam($nrtipovinculomList),
            'P_DTMESCOMPETENC'      => Utils::getParamters()->getDtmescomptencDiaCorrente(), 
        ]);

        return $std->fetchAll();
    }
}